;-------------------------------------------------------------------------------
; Mask Fade Library
; Version 1a2x1 - Modified for space dock V2
; by Teasy <TheChange@yahoo.com>
;-------------------------------------------------------------------------------
; Uses Timer System V4: See "TimerSystem 4.BB" or "TimerSystem4ms.BB"
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------

Const MaskFade_Images = 16

Dim MaskFade_Mask( 0 )

Global MaskFade_Image   ; Current mask image
Global MaskFade_Done    ; False if automatic/gradual fade is active
Global MaskFade_Target  ; -1 = out (black), 1 = in (transparent)
Global MaskFade_Timer   ; Accurate timer to track timed fade steps

;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_Init()
;-------------------------------------------------------------------------------
    Local Blend

    ; create mask images
    Dim MaskFade_Mask( MaskFade_Images )
    For Blend = 0 To MaskFade_Images
        MaskFade_Mask( Blend ) = MaskFade_CreateMaskImage( Blend )
    Next

    MaskFade_Done = True 
    ; transparent by default
    MaskFade_Image = MaskFade_Images
    MaskFade_Target = 1
    ; default 1000 ms fade duration
    MaskFade_Timer = NewTimer( 1000 / MaskFade_Images )
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_Duration( NewDuration=1000 )
;-------------------------------------------------------------------------------
; Change current speed of each fade step (16 steps total),
; using a specified duration for the entire fade.
;-------------------------------------------------------------------------------
    SetTimer MaskFade_Timer , NewDuration / MaskFade_Images
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_In()
;-------------------------------------------------------------------------------
    If MaskFade_Target = -1
        MaskFade_Target = 1
        If MaskFade_Done
            MaskFade_Done = False 
            ResetTimer MaskFade_Timer
        EndIf 
    EndIf 
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_Out()
;-------------------------------------------------------------------------------
    If MaskFade_Target = 1
        MaskFade_Target = -1
        If MaskFade_Done
            MaskFade_Done = False 
            ResetTimer MaskFade_Timer
        EndIf 
    EndIf 
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_Update()
;-------------------------------------------------------------------------------
    Local Updates
    
    ; now fading
    If Not MaskFade_Done
        ; number of elapses since last call
        Updates = TimerDone( MaskFade_Timer )
        ; any elapses
        If Updates
            ; fade in/out
            MaskFade_Image = MaskFade_Image + MaskFade_Target * Updates
            ; check if exceeding frame range
            If MaskFade_Image < 0
                ; done fading out
                MaskFade_Image = 0
                MaskFade_Done = True 
            ElseIf MaskFade_Image > MaskFade_Images
                ; done fading in
                MaskFade_Image = MaskFade_Images
                MaskFade_Done = True 
            EndIf 
        EndIf 
    EndIf 
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_Draw()
;-------------------------------------------------------------------------------
    If MaskFade_Image = 0
        ClsColor 0,0,0
        Cls 
    ElseIf MaskFade_Image < MaskFade_Images
        TileImage MaskFade_Mask( MaskFade_Image ) , Rand( 0,3 ),Rand ( 0,3 )
    EndIf 
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
Function MaskFade_CreateMaskImage( MaskType = -1 )
;-------------------------------------------------------------------------------
    Local Mask
    Local X,Y
    Local Value
    Local multiX,multiY

    ; Data mask select
    Select MaskType
        Case  0 : Restore Mask0
        Case  1 : Restore Mask1
        Case  2 : Restore Mask2
        Case  3 : Restore Mask3
        Case  4 : Restore Mask4
        Case  5 : Restore Mask5
        Case  6 : Restore Mask6
        Case  7 : Restore Mask7
        Case  8 : Restore Mask8
        Case  9 : Restore Mask9
        Case 10 : Restore Mask10
        Case 11 : Restore Mask11
        Case 12 : Restore Mask12
        Case 13 : Restore Mask13
        Case 14 : Restore Mask14
        Case 15 : Restore Mask15
        Case 16 : Restore Mask16
        Default : RuntimeError "() CreateMaskImage <- Illegal MaskType: " + MaskType
    End Select

    Mask = CreateImage( 64,64 )
    MaskImage Mask , 255,0,255
    SetBuffer ImageBuffer( Mask )

    ; Create mask
    Color 255,0,255
    For Y = 0 To 3
        For X = 0 To 3
            Read Value
            If Value
                For multiY = 0 To 15
                    For multiX = 0 To 15
                        Plot 4*multiX + X  ,  4*multiY + Y
                    Next
                Next
            EndIf
        Next
    Next

    Return Mask
;-------------------------------------------------------------------------------
End Function
;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------
; Mask Data
;-------------------------------------------------------------------------------

.Mask0  ; Opaque (black)
    Data 0,0,0,0
    Data 0,0,0,0
    Data 0,0,0,0
    Data 0,0,0,0

.Mask1
    Data 0,1,0,0
    Data 0,0,0,0
    Data 0,0,0,0
    Data 0,0,0,0

.Mask2
    Data 0,1,0,0
    Data 0,0,0,0
    Data 0,0,0,1
    Data 0,0,0,0

.Mask3
    Data 0,1,0,0
    Data 0,0,0,0
    Data 0,1,0,1
    Data 0,0,0,0

.Mask4
    Data 0,1,0,1
    Data 0,0,0,0
    Data 0,1,0,1
    Data 0,0,0,0

.Mask5
    Data 0,1,0,1
    Data 0,0,1,0
    Data 0,1,0,1
    Data 0,0,0,0

.Mask6
    Data 0,1,0,1
    Data 0,0,1,0
    Data 0,1,0,1
    Data 1,0,0,0

.Mask7
    Data 0,1,0,1
    Data 1,0,1,0
    Data 0,1,0,1
    Data 1,0,0,0

.Mask8
    Data 0,1,0,1
    Data 1,0,1,0
    Data 0,1,0,1
    Data 1,0,1,0

.Mask9
    Data 0,1,0,1
    Data 1,1,1,0
    Data 0,1,0,1
    Data 1,0,1,0

.Mask10
    Data 0,1,0,1
    Data 1,1,1,0
    Data 0,1,0,1
    Data 1,0,1,1

.Mask11
    Data 0,1,0,1
    Data 1,1,1,1
    Data 0,1,0,1
    Data 1,0,1,1

.Mask12
    Data 0,1,0,1
    Data 1,1,1,1
    Data 0,1,0,1
    Data 1,1,1,1

.Mask13
    Data 0,1,1,1
    Data 1,1,1,1
    Data 0,1,0,1
    Data 1,1,1,1

.Mask14
    Data 0,1,1,1
    Data 1,1,1,1
    Data 1,1,0,1
    Data 1,1,1,1

.Mask15
    Data 0,1,1,1
    Data 1,1,1,1
    Data 1,1,1,1
    Data 1,1,1,1

.Mask16  ; Transparent (purple)
    Data 1,1,1,1
    Data 1,1,1,1
    Data 1,1,1,1
    Data 1,1,1,1

;-------------------------------------------------------------------------------



;-------------------------------------------------------------------------------

; Self-initialize (can be called at any time)
MaskFade_Init

;-------------------------------------------------------------------------------


;-------------------------------------------------------------------------------
; Demonstration
;-------------------------------------------------------------------------------

Function MaskFade_Demo()
    Graphics 320,240 , 0,3
    MaskFade_Init
    MaskFade_Duration 1000
    SetBuffer BackBuffer()
    Repeat 
        MilliSecs = MilliSecs()
        If KeyHit( 57 )
            If MaskFade_Target = -1
                MaskFade_In
            Else 
                MaskFade_Out
            EndIf 
        EndIf 
        MaskFade_Update
        Cls 
        Color 192,224,255
        Rect 0,0 , 320,240 , False 
        Rect 320/4,240/4 , 320/2,240/2 , True 
        MaskFade_Draw
        Text 0,0 , "Image: "+MaskFade_Image
        Text 0,10, "Target: "+MaskFade_Target
        Text 0,20, "Delay: "+TimerDuration( MaskFade_Timer )
        Flip 
    Until KeyHit( 1 )
    End 
End Function

;-------------------------------------------------------------------------------
;Include "\dev\blitz\lib\timersystem 4.bb"
;Global MilliSecs
;MaskFade_Demo
;-------------------------------------------------------------------------------

