;-------------------------------------------------------------------------------
; * Generic rendering routines
; Version 2006 Oct 07 Rev 01
;-------------------------------------------------------------------------------
; Can add more as you go..!


; Draw (optional) colorized text with (optional) outline
Function ColorText( x,y , Text$ , CenterX=0,CenterY=0 , outr=-1,outg=-1,outb=-1 , separation=1 , br#=1.0,bg#=1.0,bb#=1.0 )
; * Sep 2006
; - Supports `rrrgggbbb
; - Added CenterX,CenterY (0,1,2); takes into account rgb command
; - Current color is recalled when no default color is set
; - Added variable width font support by stripping rgb command
; - Added outr=-x value for using rgb/x as outline color (goes for each outr,outg,outb)
;   for example -2 means use half rgb for outline color
; * Oct 2006
; - Added br,bg,bb: global brightness (0..1)
; - Optimizing: Draw per color section (skip to next `)
;   * Algorithm
;       PrevCharNum = 1
;       Repeat
;           CharNum = Instr( Text , "`" , PrevCharNum )
;           ; found a next color header
;           If CharNum > 0
;               ; grab (actual) text inbetween previous and next color header
;               Section = Mid( Text , PrevCharNum , CharNum-PrevCharNum )  ;"`123456789abc`123456789" 11,14->11,3
;               * DRAW (with offset advance)
;               * COLOR (with string parsing)
;               CharNum = CharNum + 3+3+3+1  ; skip over `rrrgggbbb at next char
;               PrevCharNum = CharNum
;           ; didn't find a next color header
;           Else
;               ; still something left to draw (tail)
;               If PrevCharNum <= Length
;                   Section = Mid( Text , PrevCharNum , Length-PrevCharNum+1 )  ;"abc`123456789abc" 14,16->14,3
;                   * DRAW
;               EndIf
;               ; all done
;               Exit
;           EndIf
;       Forever
; - Fixed original color ability
;   you can write something without color header, using current color, e.g. outside of function
    Local CharNum,Char$,Length
    Local r,g,b,ox,oxb,oy
    Local Raw$,outdiv
    Local Section$,PrevCharNum
    Local startr,startg,startb
    ; precalc length and set default color
    Length = Len( Text )
    startr = ColorRed() * br
    startg = ColorGreen() * bg
    startb = ColorBlue() * bb
    ; Strip `rrrgggbbb
    PrevCharNum = 1
    Repeat
        CharNum = Instr( Text , "`" , PrevCharNum )
        ; found a next color header
        If CharNum > 0
            ; grab (actual) text inbetween previous and next color header
            Raw = Raw + Mid( Text , PrevCharNum , CharNum-PrevCharNum )
            CharNum = CharNum + 3+3+3+1  ; skip over `rrrgggbbb at next char
            PrevCharNum = CharNum
        ; didn't find a next color header
        Else
            ; still something left to draw (tail)
            If PrevCharNum <= Length
                Raw = Raw + Mid( Text , PrevCharNum , Length-PrevCharNum+1 )
            EndIf
            ; all done
            Exit
        EndIf
    Forever
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If CenterX = 2
        ox = -StringWidth( Raw )
    ElseIf CenterX = 1
        ox = -StringWidth( Raw ) / 2
    Else
        ox = 0
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If CenterY = 2
        oy = -StringHeight( Raw )
    ElseIf CenterY = 1
        oy = -StringHeight( Raw ) / 2
    Else
        oy = 0
    EndIf
    ; Format and render outline (if used)
    If outr <> -1
        ; use default color
        r = startr
        g = startg
        b = startb
        ; offset x background (increased per section)
        oxb = ox
        PrevCharNum = 1
        Repeat
            CharNum = Instr( Text , "`" , PrevCharNum )
            ; found a next color header
            If CharNum > 0
                ; grab (actual) text inbetween previous and next color header
                Section = Mid( Text , PrevCharNum , CharNum-PrevCharNum )  ;"`123456789abc`123456789" 11,14->11,3
                If outr > -1  ; specific color
                    ; Draw outline only (if specified)
                    TextSpread oxb+x,oy+y , Section , False,False , outr*br,outg*bg,outb*bb , separation
                ElseIf outr < -1  ; division of current color
                    ; Draw outline only (using rgb/-x)
                    ;TextSpread oxb+x,oy+y , Section , False,False , (r/Float -outr)*br,(g/Float -outg)*bg,(b/Float -outb)*bb , separation
                    TextSpread oxb+x,oy+y , Section , False,False , (r/-outr)*br,(g/-outg)*bg,(b/-outb)*bb , separation
                EndIf
                oxb = oxb + StringWidth( Section )
                If outr < -1
                    r = Mid( Text , CharNum+1,3 )  ; `rrr000000
                    g = Mid( Text , CharNum+4,3 )  ; `000ggg000
                    b = Mid( Text , CharNum+7,3 )  ; `000000bbb
                    Color r*br,g*bg,b*bb
                EndIf
                CharNum = CharNum + 3+3+3+1  ; skip over `rrrgggbbb at next char
                PrevCharNum = CharNum
            ; didn't find a next color header
            Else
                ; still something left to draw (tail)
                If PrevCharNum <= Length
                    Section = Mid( Text , PrevCharNum , Length-PrevCharNum+1 )  ;"abc`123456789abc" 14,16->14,3
                    If outr > -1
                        ; Draw outline only (if specified)
                        TextSpread oxb+x,oy+y , Section , False,False , outr*br,outg*bg,outb*bb , separation
                    ElseIf outr < -1
                        ; Draw outline only (using rgb/-x)
                        ;TextSpread oxb+x,oy+y , Section , False,False , (r/Float -outr)*br,(g/Float -outg)*bg,(b/Float -outb)*bb , separation
                        TextSpread oxb+x,oy+y , Section , False,False , (r/-outr)*br,(g/-outg)*bg,(b/-outb)*bb , separation
                    EndIf
                EndIf
                ; all done
                Exit
            EndIf
        Forever
    EndIf
    ; use default color
    r = startr
    g = startg
    b = startb
    Color r,g,b
    ; Format and render overlay (front)
    PrevCharNum = 1
    Repeat
        CharNum = Instr( Text , "`" , PrevCharNum )
        ; found a next color header
        If CharNum > 0
            ; grab (actual) text inbetween previous and next color header
            Section = Mid( Text , PrevCharNum , CharNum-PrevCharNum )  ;"`123456789abc`123456789" 11,14->11,3
            Text ox+x,oy+y , Section
            ox = ox + StringWidth( Section )
            r = Mid( Text , CharNum+1,3 )  ; `rrr000000
            g = Mid( Text , CharNum+4,3 )  ; `000ggg000
            b = Mid( Text , CharNum+7,3 )  ; `000000bbb
            Color r*br,g*bg,b*bb
            CharNum = CharNum + 3+3+3+1  ; skip over `rrrgggbbb at next char
            PrevCharNum = CharNum
        ; didn't find a next color header
        Else
            ; still something left to draw (tail)
            If PrevCharNum <= Length
                Section = Mid( Text , PrevCharNum , Length-PrevCharNum+1 )  ;"abc`123456789abc" 14,16->14,3
                Text ox+x,oy+y , Section
            EndIf
            ; all done
            Exit
        EndIf
    Forever
End Function


; Like Text() only with fixed width characters
Function CharText( x,y , msg$ , cx=False,cy=False , r=-1,g=-1,b=-1 , ccw=-1,cch=-1 )
; ccw,cch: (optional) custom char width/height
    Local cw,ch
    Local length,char

    If ccw = -1
        cw = StringWidth( "A" )
    Else 
        cw = ccw
    EndIf 
    If cch = -1
        ch = StringHeight( "A" )
    Else 
        ch = cch
    EndIf 

    length = Len( msg )

    ; draw all chars from middle of char
    x = x + cw / 2
    y = y + ch / 2
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If cx = 2
        x = x - cw * length;StringWidth( msg )
    ElseIf cx = 1
        x = x - cw * length / 2;StringWidth( msg ) / 2
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If cy = 2
        y = y - ch;StringHeight( msg )
    ElseIf cy = 1
        y = y - ch / 2;StringHeight( msg ) / 2
    EndIf
    If b > -1
        Color r,g,b
    EndIf 
    For char = 1 To length
        Text x,y , Mid( msg , char,1 ) , True,True 
        x = x + cw
    Next 
End Function


; Like Text() only with 3x3 outline, for example to make white text visible on white background
Function TextOutLine( PosX%,PosY% , Message$ , CenterX=False,CenterY=False , InR=0,InG=0,InB=0 , OutR=255,OutG=255,OutB=255 , Separation=1 )
    Local X%
    Local Y%
    Color OutR,OutG,OutB
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If CenterX = 2
        PosX = PosX - StringWidth( Message )
        CenterX = False
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If CenterY = 2
        PosY = PosY - StringHeight( Message )
        CenterY = False
    EndIf
    For X = -1 To 1
        For Y = -1 To 1
            Text PosX + X * Separation , PosY + Y * Separation , Message , CenterX , CenterY
        Next
    Next
    Color InR,InG,InB
    Text PosX,PosY , Message , CenterX,CenterY
End Function


; Like TextOutline() only with 1 color
Function TextSpread( PosX%,PosY% , Message$ , CenterX=False,CenterY=False , R=255,G=0,B=0 , Separation=1 )
    Local X%
    Local Y%
    Color R,G,B
    ; CenterX: 0 = Left justify, 1 = Center, 2 = Right justify
    If CenterX = 2
        PosX = PosX - StringWidth( Message )
        CenterX = False
    EndIf
    ; CenterY: 0 = Top justify, 1 = Center, 2 = Bottom justify
    If CenterY = 2
        PosY = PosY - StringHeight( Message )
        CenterY = False
    EndIf
    For X = -1 To 1
        For Y = -1 To 1
            Text PosX + X * Separation , PosY + Y * Separation , Message , CenterX , CenterY
        Next
    Next
End Function


; A filled and shaded rectangle with upper-left light and lower-right darkness
Function drawbevel( x,y , w,h , r,g,b , bright#=1.0,medium#=0.7,dark#=0.4 )
    Local br,bg,bb
    Local mr,mg,mb
    Local dr,dg,db
    br=r*bright:bg=g*bright:bb=b*bright
    mr=r*medium:mg=g*medium:mb=b*medium
    dr=r*dark  :dg=g*dark  :db=b*dark
    Color mr,mg,mb
    Rect x+1,y+1 , w-2,h-2 , True
    Plot x,y+h-1
    Plot x+h-1,y
    Color br,bg,bb
    Rect x,y , w-1,1 , True
    Rect x,y , 1,h-1 , True
    Color dr,dg,db
    Rect x+1,y+h-1 , w-1,1 , True
    Rect x+w-1,y+1 , 1,h-1 , True
End Function


; Draw a normal Rect, only with extra outline around/inside it
Function rectoutline( x,y , w,h , solid=False , fr=-1,fg=-1,fb=-1 , br=-1,bg=-1,bb=-1 )
; x,y: start location
; w,h: width,height
; solid: draw filled or frame
; fr,fg,fb: foreground (inside) red,green,blue
; br,bg,bb: background (outline) red,green,blue
; background (outline) is not drawn if back color not specified
; if foreground color is not specified, current color is used
    usefront = fr >= 0 And fg >= 0 And fb >= 0
    useback  = br >= 0 And bg >= 0 And bb >= 0
    ; back color specified
    If useback
        ; front color NOT specified
        If Not usefront
            ; remember current (front) color
            r = ColorRed()
            g = ColorGreen()
            b = ColorBlue()
            ; draw back
            Color br,bg,bb
            Rect x,y , w,h , False
            ; draw inner outline if not solid
            If Not solid
                Rect x+2,y+2 , w-4,h-4 , False 
            EndIf 
            ; recall front color
            Color r,g,b
        ; front color specified
        Else
            ; draw back
            Color br,bg,bb
            Rect x,y , w,h , False
            ; draw inner outline if not solid
            If Not solid
                Rect x+2,y+2 , w-4,h-4 , False 
            EndIf 
            ; select front color
            Color fr,fg,fb
        EndIf
    ; back color NOT specified -> don't draw
    Else
        ; front color specified
        If usefront
            ; select front color
            Color fr,fg,fb
        EndIf
    EndIf
    ; draw front
    Rect x+1,y+1 , w-2,h-2 , solid
End Function


; Same as RectOutline, only with Oval
Function ovaloutline( x,y , w,h , solid=False , fr=-1,fg=-1,fb=-1 , br=-1,bg=-1,bb=-1 )
    usefront = fr >= 0 And fg >= 0 And fb >= 0
    useback  = br >= 0 And bg >= 0 And bb >= 0
    If useback
        If Not usefront
            r = ColorRed()
            g = ColorGreen()
            b = ColorBlue()
            Color br,bg,bb
            Oval x,y , w,h , False
            If Not solid
                Oval x+2,y+2 , w-4,h-4 , False 
            EndIf 
            Color r,g,b
        Else
            Color br,bg,bb
            Oval x,y , w,h , False
            If Not solid
                Oval x+2,y+2 , w-4,h-4 , False 
            EndIf 
            Color fr,fg,fb
        EndIf
    Else
        If usefront
            Color fr,fg,fb
        EndIf
    EndIf
    Oval x+1,y+1 , w-2,h-2 , solid
End Function

