;-------------------------------------------------------------------------------
; * String Library
; Version 2007 Jun 15
; Created by Teasy <TheChange@yahoo.com>
;-------------------------------------------------------------------------------
; General purpose text handling and manipulation routines.
;-------------------------------------------------------------------------------
; Warning:
;   Blitz\Trim() removes spaces and tabs,
;   but does not support extended characters.
; Solution:
;   String_Trim() does support extended characters,
;   but is slower and doesn't remove tabs, but is very safe.
;-------------------------------------------------------------------------------
; For function usage, see each function's individual description.
;-------------------------------------------------------------------------------


Const String_NumberList$ = "1234567890"
Const String_AlphaList$ = "ABCDEFGHIJKLMNOPQRSTUVWXYZ" + "abcdefghijklmnopqrstuvwxyz" + "" + ""
Global String_SignList$ = "!"+Chr(34)+"#$%&'()*+,-./:;<=>?" + "@[\]^_`{|}~" + ""
Global String_SpaceList$ = Chr(9)+Chr(32)+Chr(160)


;-------------------------------------------------------------------------------
;--> Files
;-------------------------------------------------------------------------------


; Add trailing backslash, if not present
Function String_TrailingBackslash$( path$ )
    If Right( path , 1 )<>"\"
        Return path+"\"
    Else
        Return path
    EndIf
End Function


; Wipe trailing backslash, if present
Function String_RemoveTrailingBackslash$( path$ )
    If Right( path , 1 )="\"
        Return Left( path , Len( path )-1 )
    Else
        Return path
    EndIf
End Function


; Keep only file name, strip path and drive (if used)
; - Cuts everything before last "\" or first ":" respectively
Function String_GetFileName$( LookIn$ , Separator$="\" )
    Local Pos
    Local Length
    Local Where

    Length = Len( LookIn )
    ; Folder separator found
    If Instr( LookIn , Separator )
        ; So filename starts after last separator
        For Pos = Length To 1 Step -1
            If Mid( LookIn , Pos,1 ) = Separator
                Return Right( LookIn , Length-Pos )
            EndIf
        Next
    ; No separator found
    Else
        ; Strip drive (if present)
        Where = Instr( LookIn , ":" )
        If Where
            Return Right( LookIn , Length-Where )
        EndIf
    EndIf

    ; No separator, no drive, no change
    Return LookIn
End Function


; Strip file name from absolute file location
; - Cuts everything after last "\" or first ":" respectively
Function String_GetPathName$( FileSpec$ , Separator$="\" )
    Local Char
    Local Length
    Local Where

    Length = Len( FileSpec )
    ; Folder separator found
    If Instr( FileSpec , Separator )
        ; So cut off everything behind last separator
        For Char = Length To 1 Step -1
            If Mid( FileSpec , Char,1 ) = Separator
                Return Left( FileSpec , Char )
            EndIf
        Next
    ; No folder separator found
    Else
        ; Maybe there's a drive
        Where = Instr( FileSpec , ":" )
        If Where
            ; Keep only the drive
            Return Left( FileSpec , Where )
        EndIf
    EndIf

    ; No folders, no drive, no nothing :)
    Return ""
End Function


;-------------------------------------------------------------------------------
;--> Conversion
;-------------------------------------------------------------------------------


; Reverse string (e.g. "123" --> "321")
Function String_Flip$( In$ )
    Local Length
    Local Pos
    Local Out$

    Length = Len( In )
    For Pos = Length To 1 Step -1
        Out = Out + Mid( In , Pos,1 )
    Next

    Return Out
End Function


; Convert a boolean (or tri-boolean) value to a text string
; using a dynamic string parameter
Function String_BooleanText$( BooleanValue , Format$="" , NumParams=2 )
; BooleanValue: (normal) True, False or 'Neither'
;               (custom) 0, 1, .. NumParams (default = 2/normal)
; BooleanValue is 0-based when used to parse format parameters
; NumParams: specifies how many sections in Format, e.g. "ZeroOne Two " = 3
; Custom format is used when a predefined format is not identified (see below)
; Example: ( 2 , "FalseTrue Eh?" , 3 ) = "Eh?"
; Warning: uses TRIM, cuts extended characters
    Local Value

    If BooleanValue < 0 Or BooleanValue > 1
        Value = 2
    Else
        Value = BooleanValue
    EndIf
    Select Format
        Case 0,"True","False",""
            Return Trim( Mid( "FalseTrue = ! =" , Value*5+1 , 5 ) )
        Case 1,"Enabled","Disabled"
            Return Trim( Mid( "DisabledEnabled Absonant" , Value*8+1 , 8 ) )
        Case 2,"Yes","No"
            Return Trim( Mid( "No Yes=!=" , Value*3+1 , 3 ) )
        Case 3,"Off","On"
            Return Trim( Mid( "OffOn =!=" , Value*3+1 , 3 ) )
        Default
            ;line1$ = "Format not supported"+Chr(13)
            ;line2$ = "fn String_BooleanText$"+Chr(13)
            ;line3$ = "BooleanValue% = "+BooleanValue+Chr(13)
            ;line4$ = "Format$ = "+Format
            ;RuntimeError line1+line2+line3+line4
            ; Custom format:
            Return Trim( Mid( Format , BooleanValue*(1+Len( Format ))/NumParams+1 , (1+Len( Format ))/NumParams ) )
    End Select
End Function


; Return color as hexadecimal space-separated format (e.g. "00 8F FF")
Function String_FormatColor$ ( Red , Green , Blue )
    Return Right ( Hex ( Red ) , 2 ) + " " + Right ( Hex ( Green ) , 2 ) + " " + Right ( Hex ( Blue ) , 2 )
End Function


; Add commas to value (e.g. $ 15,000 )
Function String_WithCommas$ ( Value$ )
    Local Counter
    Local Count
    Local Result$

    Result = Value
    For Counter = Len ( Result ) To 1 Step -1
        If Count = 3
            Result = Left ( Result , Counter ) + "," + Right ( Result , Len ( Result ) - Counter )
            Count = 0
        End If
        Count = Count + 1
    Next

    Return Result
End Function


; Return auto-scaled byte formatted value
Const String_FormatBytes_Limit1   =        999
Const String_FormatBytes_Limit2   =     999999
Const String_FormatBytes_Limit3   =  999999999
Const String_FormatBytes_Divisor1 =       1024
Const String_FormatBytes_Divisor2 =    1048576
Const String_FormatBytes_Divisor3 = 1073741824
Function String_FormatBytes$( Value )
    Local ReturnValue$
    Local ReturnUnits$
    Local Signif%
    Local Negative

    If Value < 0
        Negative = True
        Value = Abs( Value )
    EndIf
    If Value < String_FormatBytes_Limit1
        ReturnValue = Value
        ReturnUnits = ""
    ElseIf Value < String_FormatBytes_Limit2
        Signif = 10 ^ ( 3 - Len( Value / String_FormatBytes_Divisor1 ) )
        ReturnValue = Float Int( Float Value * Signif / String_FormatBytes_Divisor1 ) / Signif
        ReturnUnits = " K"
    ElseIf Value < String_FormatBytes_Limit3
        Signif = 10 ^ ( 3 - Len( Value / String_FormatBytes_Divisor2 ) )
        ReturnValue = Float Int( Float Value * Signif / String_FormatBytes_Divisor2 ) / Signif
        ReturnUnits = " M"
    Else
        Signif = 10 ^ ( 3 - Len( Value / String_FormatBytes_Divisor3 ) )
        ReturnValue = Float Int( Float Value * Signif / String_FormatBytes_Divisor3 ) / Signif
        ReturnUnits = " G"
    EndIf
    If Len( ReturnValue ) > 2
        If Right( ReturnValue , 2 ) = ".0"
            ReturnValue = Left( ReturnValue , Len( ReturnValue ) - 2 )
        EndIf
    EndIf

    If Negative
        Return "-" + ReturnValue + ReturnUnits
    Else
        Return ReturnValue + ReturnUnits
    EndIf
End Function


;-------------------------------------------------------------------------------
;--> Clipping
;-------------------------------------------------------------------------------


; Wipe leading and trailing spaces (nothing else, like tabs)
Function String_Trim$( Source$ )
    Local CharL,CharR
    Local Length

    Length = Len( Source )
    ; if first char is a space
    If Left( Source , 1 ) = " "
        ; find first non-space
        For CharL = 1 To Length
            If Mid( Source , CharL,1 ) <> " "
                Exit
            EndIf
        Next
    Else
        CharL = 1
    EndIf
    ; if last char is a space
    If Right( Source , 1 ) = " "
        ; find last non-space
        For CharR = Length To 1 Step -1
            If Mid( Source , CharR,1 ) <> " "
                Exit
            EndIf
        Next
    Else
        CharR = Length
    EndIf

    ; trim left/right
    Return Mid( Source , CharL,CharR-CharL+1 )
End Function


; Remove number of leading chars
Function String_WipeLeft$( Source$ , AmountOfChars )
    Return Mid( Source , AmountOfChars+1 , Len( Source )-AmountOfChars )
End Function


; Remove any leading number of occurences
Function String_WipeLeading$( This$ , Lead$ )
    While Left( This , Len( Lead ) ) = Lead
        This = Right( This , Len( This )-Len( Lead ) )
    Wend

    Return This
End Function


; Remove Tail section (single char or string) from This
Function String_WipeTrailing$( This$ , Tail$ )
    While Right( This , Len( Tail ) ) = Tail
        This = Left( This , Len( This )-Len( Tail ) )
    Wend

    Return This
End Function


; Return trailing section of string
Function String_RightOf$( Source$ , WhereFrom )
    Return Right( Source , Len( Source )-WhereFrom+1 )
End Function


; Remove quotes if present
Function String_CutQuotes$( Source$ )
    Local TrimSource$

    TrimSource = String_Trim( Source )
    If String_LeftIs( TrimSource , Chr(34) ) And String_RightIs( TrimSource , Chr(34) )
        Return Mid( TrimSource , 2,Len( TrimSource )-2 )
    Else
        Return Source
    End If
End Function


; Remove outer quotes and 'singlefy' any inner quotes
Function String_SimplifyQuotes$( Source$ )
    Local TrimSource$

    TrimSource = String_Trim( Source )
    If String_LeftIs( TrimSource , Chr(34) ) And String_RightIs( TrimSource , Chr(34) )
        Return Replace( Mid( TrimSource , 2,Len( Trim( Source ) )-2 ) , Chr(34)+Chr(34) , Chr(34) )
    Else
        Return Replace( Source , Chr(34)+Chr(34) , Chr(34) )
    End If
End Function


;-------------------------------------------------------------------------------
;--> Comparison
;-------------------------------------------------------------------------------


; Return number of chars found in string
Function String_CountChars( Source$ , FindChar$ )
    Local Where
    Local HowMany

    ; Find first occurence
    Where = Instr( Source , FindChar )
    ; Continue on any occurence
    While Where > 0
        ; Increase number of occurences
        HowMany = HowMany + 1
        ; Find next occurence
        Where = Instr( Source , FindChar , Where+1 )
    Wend

    ; Return number of occurences
    Return HowMany
End Function


; Match trailing section of string
Function String_RightIs( Source$ , CompareWith$ )
    Return Right( Source , Len( CompareWith ) ) = CompareWith
End Function


; Match leading section of string
Function String_LeftIs( Source$ , CompareWith$ )
    Return Left( Source , Len( CompareWith ) ) = CompareWith
End Function


; Match section in string
Function String_MidIs ( MainText$ , Position% , SubText$ )
    Return Mid ( MainText , Position , Len ( SubText ) ) = SubText
End Function


; Check if string is an integer
; - Also invalid when integer overflows
Function String_IsNumber ( This$ )
    Local Number
    Local Clean$

    ; convert to number
    Number = This$
    ; multiple characters
    If Len( This ) > 1
        ; wipe leading zeroes
        Clean = String_WipeLeading( This , "0" )
        ; check if removed entire string
        If Len( Clean ) < 1
            ; put a single zero
            Clean = "0"
        EndIf
    ; single character
    Else
        ; use straight-up
        Clean = This
    End If

    ; match converted number with cleaned string
    If Number = Clean
        Return True
    EndIf
End Function


; Check if specified character is an Alpha character
Function String_IsAlpha( Char$ )
    Return Instr( String_AlphaList , Char ) > 0
End Function


; Check if specified character is an Alpha or Numeric character
Function String_IsAlphaNum( Char$ )
    Return Instr( String_AlphaList , Char ) > 0 Or Instr( String_NumberList , Char ) > 0
End Function


; Check if specified character is a Sign character
Function String_IsSign( Char$ )
    Return Instr( String_SignList , Char ) > 0
End Function


; Check if specified character is a Space character
Function String_IsSpace( Char$ )
    Return Instr( String_SpaceList , Char ) > 0
End Function


; Return position of first occurence of space to the left of position
Function String_SpaceFromHere ( Text$ , Where% , Space$ = " " )
    Local Char
    Local Length

    Length = Len ( Text )
    For Char = Where To Length
        If Mid ( Text , Char , 1 ) = Space Then Return Char
    Next
End Function


; Return position of first occurence of space to the right of position
Function String_SpaceBackThere ( Text$ , Where% , Space$ = " " )
    Local Char

    For Char = Where To 1 Step -1
        If Mid ( Text , Char , 1 ) = Space Then Return Char
    Next
End Function


;-------------------------------------------------------------------------------
;--> Addition
;-------------------------------------------------------------------------------


; Return amount of random chars
Function String_GenerateGarbage$( Width=0 , Min=32 , Max=255 )
    Local n
    Local garb$
    Local length

    If Width>0  ; Strict amount
        For n=1 To Width
            garb$=garb+Chr( Rand( min,max ) )
        Next
    ElseIf Width<0  ; Random amount
        length=Rand( 1,-Width )
        For n=1 To length
            garb=garb+Chr( Rand( min,max ) )
        Next
    EndIf

    Return garb
End Function


; Add leading string to match space
; - Cuts value if too long!
Function String_PadValue$ ( Value , Length , Fill$="0" )
    Local ValueLength

    ValueLength = Len ( Value )
    If ValueLength = Length
        Return Value  ; Keep
    Else If ValueLength > Length
        Return Right ( Value , Length )  ; Cut
    Else If ValueLength < Length
        Return String ( Fill , Length - ValueLength ) + Value  ; Add
    End If
End Function


; Add leading string to match space
; - Does not cut value if too long
Function String_PadLeft$ ( Text$ , PadAmount , PadChar$ = " " )
    If Len ( Text ) >= PadAmount
        Return Text
    Else
        Return String ( PadChar , PadAmount - Len ( Text ) ) + Text
    End If
End Function


; Add trailing string to match space
; - Does not cut value if too long
Function String_PadRight$ ( Text$ , PadAmount , PadChar$ = " " )
    If Len ( Text ) >= PadAmount
        Return Text
    Else
        Return Text + String ( PadChar , PadAmount - Len ( Text ) )
    End If
End Function


;-------------------------------------------------------------------------------
;--> Parameters
;-------------------------------------------------------------------------------


; Wipe leading space-separated parameter from string
Function String_CutFirstParameter$ ( Source$ )
    Return String_Trim( Right( Source , Len( Source )-Len( String_Parameter( Source , 1 ) ) ) )
End Function


; Return specific parameter from parameter separated string
Function String_Parameter$ ( From$ , Which% , Space$ = " " , Quote$ = "" )  ; Original by skn3[ac]
    Local foundword=False
    Local mode=False
    Local ignorespace=False
    Local current=0
    Local maketok$=""
    Local getchar$=""
    Local i

    If Quote = "" Then Quote = Chr ( 34 )
    For i=1 To Len(from$)
        getchar$=Mid$(from$,i,1)
        If getchar=Quote Then ignorespace=Not ignorespace
        If foundword=False Then
            If mode=False Then
                If getchar$<>space$ Or ignorespace=True Then
                    mode=True
                    current=current+1
                End If
                If current=which Then
                    foundword=True
                    maketok$=maketok$+getchar$
                End If
            Else
                If getchar$=space$ And ignorespace=False Then
                    mode=False
                End If
            End If
        Else
            If getchar$=space$ And ignorespace=False Then
                Exit
            Else
                maketok$=maketok$+getchar$
            End If
        End If
    Next

    Return maketok$
End Function


; Return total number of separated parameters in string
Function String_ParameterCount ( From$ , Space$ = " " , Quote$ = "" )  ; Original by skn3[ac]
    Local mode
    Local ignorespace
    Local current
    Local getchar$
    Local i

    If Quote = "" Then Quote = Chr ( 34 )
    For i=1 To Len(from$)
        getchar$=Mid$(from$,i,1)
        If getchar=Quote Then ignorespace=Not ignorespace
        If mode=False
            If getchar$<>space$ Or ignorespace=True
                mode=True
                current=current+1
            End If
        Else
            If getchar$=space$ And ignorespace=False
                mode=False
            End If
        End If
    Next

    Return current
End Function


; Returns true if any separated parameter is identical to search string
Function String_ContainsParameter ( Source$ , Parameter$ , Separator$ = " " , Quote$ = "" )
; Note: slow function due to extreme encapsulation
    Local Count
    Local Number

    Count = String_ParameterCount ( Source , Separator , Quote )
    For Number = 1 To Count
        If String_Parameter ( Source , Number , Separator , Quote ) = Parameter
            Return True
        End If
    Next
End Function


; Return position of specific parameter in parameter separated string
Function String_ParameterPos( From$ , Which% , Space$ = " " , Quote$ = "" )  ; Original by skn3[ac]
    Local foundword=False
    Local mode=False
    Local ignorespace=False
    Local current=0
    Local maketok$=""
    Local getchar$=""
    Local i

    If Quote = "" Then Quote = Chr ( 34 )
    For i=1 To Len(from$)
        getchar$=Mid$(from$,i,1)
        If getchar=Quote Then ignorespace=Not ignorespace
        If foundword=False Then
            If mode=False Then
                If getchar$<>space$ Or ignorespace=True Then
                    mode=True
                    current=current+1
                End If
                If current=which Then
                    Return i
                    foundword=True
                    maketok$=maketok$+getchar$
                End If
            Else
                If getchar$=space$ And ignorespace=False Then
                    mode=False
                End If
            End If
        Else
            If getchar$=space$ And ignorespace=False Then
                Exit
            Else
                maketok$=maketok$+getchar$
            End If
        End If
    Next

    Return maketok$
End Function


