;-----------------------------------------------------------
; Vector Collision Library (VCL)
;-----------------------------------------------------------
; Version 2.07/07/03.1 (Built on VML 7.2)
; Created by Teasy <TheChange@yahoo.com>
; Copyright (C) 2001-2007. All rights reserved.
;-----------------------------------------------------------

;-----------------------------------------------------------
; Functions
;-----------------------------------------------------------
; * CreateVectorModel.VectorModel (
;       Source = -1 , File$ = "" ,
;       HotSpotX# = -1 , HotSpotY# = -1 ,
;       MaskRed = 0 , MaskGreen = 0 , MaskBlue = 0 ,
;       Precision = 1 , Closed = True , Frame = 0
;   )
;
;   Source can be any of the following:
;       Source_ImageFile: Load bitmap from file
;       Source_ImageHandle: Use bitmap in memory (File = handle)
;                           Note: Frame = which frame if anim image
;       Source_Data: Load data from source (use Restore)
;       Source_File: Load data from file (binary, use Editor)
;
;   Precision means skipping value for points;
;       1 = all points (best precision)
;       max (numPoints-2) = 2 points total
;
;   Any vector model/image containing less than 3 points
;   will be an illegal vector model for usage.
;-----------------------------------------------------------
; * VectorModelsCollide (
;       Model1.VectorModel ,
;       Model1_PosX , Model1_PosY , Model1_Angle ,
;       Model2.VectorModel ,
;       Model2_PosX , Model2_PosY , Model2_Angle ,
;       Model1_Scale# = 1 , Model2_Scale# = 1
;   )
;-----------------------------------------------------------
; * DrawVectorModel (
;       Model.VectorModel ,
;       ModelX , ModelY ,
;       ModelAngle = 0 , ModelScale# = 1
;   )
;-----------------------------------------------------------
; * DetectVectorModelCollisionRadius (
;       Model.VectorModel
;   )
;
;   Use the returned radius as a minimum collision distance
;   before testing for precise collisions.
;-----------------------------------------------------------
; - there are more useful functions located below
; - most of the new functions are left-aligned

;===========================================================









;===========================================================

;-------------------------------------------------------------------------------
; * 2D Vector Library
; Ver 2004 Jul 03 Rev 1
;
;                  &     &&&&& &&&&  &&&&   &&&  &&&&  &   &
;                  &:     :&:::&:::& &:::& & ::& &:::& &:  &:
;                  &:      &:  &&&& :&&&& :&&&&&:&&&& : &&& :
;                  &:      &:  &:::& &:::& &:::&:&:::&   &::
;                  &&&&& &&&&& &&&& :&:  &:&:  &:&:  &:  &:
;                   ::::: ::::: ::::  :   : :   : :   :   :
;
;  Created by TheChance.
;  With the profound help of many experienced people over on BlitzCoder.
;  Just to name a few: Eelco, DarkEagle, Foppy, ObviousDisaster and Antony.
;-------------------------------------------------------------------------------
; * Premise
;
;  Although you can draw nice vector models with this lib, the primary goal
;  of this lib is to be able to use 3D sprites with pixel-perfect collisions.
;-------------------------------------------------------------------------------
; * Primary functions
;
;     * Using vector models (convex hulls or collections of vertices)
;         * to accurately detect collisions.
;         * for display.
;     * Multi-layered collision system including
;         * dividing a world into sectors
;         * "aabb" motion (collision box)
;         * sphere sweep (collision radius)
;         * vector model collision (precise)
;-------------------------------------------------------------------------------
; * How it works
;
;  The typical example is when you have a simple 2D image,
;  and you want to be able to not only rotate it with high accuracy,
;  but also zoom it in till the whole screen is consumed with huge pixels,
;  and still have accurate collisions, all the way down to the pixel level.
;
;  First, grab an image and throw it into the vector model converter.
;
;    1.  Image -> Vector model
;
;  Now you can use this vector model for collisions.
;
;    2.  Vector models -> Collision detection
;
; etc :)
;
; (oct 2k5) v2 draft:
;       take a 2d image you want to rotate and zoom
;       etc
;   story is nice for the reader
;
;-------------------------------------------------------------------------------
; * Functions
;
;-------------------------------------------------------------------------------

Const Dir_None      = 0
Const Dir_UpLeft    = 1
Const Dir_Up        = 2
Const Dir_UpRight   = 3
Const Dir_Right     = 4
Const Dir_DownRight = 5
Const Dir_Down      = 6
Const Dir_DownLeft  = 7
Const Dir_Left      = 8
Const Dir_Illegal   = 9

Type Tracer_Point
    Field X
    Field Y
End Type


Function Tracer_PointCreate.Tracer_Point ( X = 0 , Y = 0 )
    Local ptr.Tracer_Point

    ptr = New Tracer_Point
    ptr\X = X
    ptr\Y = Y

    Return ptr
End Function


Function Tracer_PointCount ()
    Local ptr.Tracer_Point
    Local Count

    For ptr = Each Tracer_Point
        Count = Count + 1
    Next

    Return Count
End Function


Function Dir_RotateRight ( Dir )
    If Dir = Dir_Left
        Return Dir_UpLeft
    Else
        Return Dir + 1
    End If
End Function


Function Dir_IsSame ( p.Tracer_Point[ 3 ] )
    Local Dir_1to2
    Local Dir_2to3

    Dir_1to2 = VectorToDir ( Sgn ( p[ 2 ]\X - p[ 1 ]\X ) , Sgn ( p[ 2 ]\Y - p[ 1 ]\Y ) )
    If Dir_1to2 = Dir_Illegal Then Return False

    Dir_2to3 = VectorToDir ( Sgn ( p[ 3 ]\X - p[ 2 ]\X ) , Sgn ( p[ 3 ]\Y - p[ 2 ]\Y ) )
    If Dir_2to3 = Dir_Illegal Then Return False

    Return Dir_1to2 = Dir_2to3
End Function


Function VectorToDir ( DirX , DirY )
    If DirX = 0 And DirY = 0
        Return Dir_None
    Else If DirX = -1 And DirY = -1
        Return Dir_UpLeft
    Else If DirX = 0 And DirY = -1
        Return Dir_Up
    Else If DirX = 1 And DirY = -1
        Return Dir_UpRight
    Else If DirX = 1 And DirY = 0
        Return Dir_Right
    Else If DirX = 1 And DirY = 1
        Return Dir_DownRight
    Else If DirX = 0 And DirY = 1
        Return Dir_Down
    Else If DirX = -1 And DirY = 1
        Return Dir_DownLeft
    Else If DirX = -1 And DirY = 0
        Return Dir_Left
    Else
        Return Dir_Illegal
    End If
End Function


Function DirToVector.Tracer_Point ( Dir )
    Local Point.Tracer_Point

    Point = New Tracer_Point
    Select Dir
        Case Dir_UpLeft
            Point\X = -1
            Point\Y = -1
        Case Dir_Up
            Point\X = 0
            Point\Y = -1
        Case Dir_UpRight
            Point\X = 1
            Point\Y = -1
        Case Dir_Right
            Point\X = 1
            Point\Y = 0
        Case Dir_DownRight
            Point\X = 1
            Point\Y = 1
        Case Dir_Down
            Point\X = 0
            Point\Y = 1
        Case Dir_DownLeft
            Point\X = -1
            Point\Y = 1
        Case Dir_Left
            Point\X = -1
            Point\Y = 0
        Default
            Delete Point
    End Select
    Return Point
End Function

;-------------------------------------------------------------------------------

Global Tracer_Image_SizeX
Global Tracer_Image_SizeY

Dim Tracer_Image( 0 , 0 )


Function Tracer_ConvertImageToArray ( File$ , LoadFromFile = True , Frame = 0 )
    Local Image_Handle
    Local PosX
    Local PosY

    If LoadFromFile
        Image_Handle = LoadImage ( File )
        If Not Image_Handle Then RuntimeError "[VectorLib]" + Chr(13) + "Tracer_ConvertImageToArray ()" + Chr(13) + "Image format not supported:" + Chr(13) + File
    Else 
        Image_Handle = Int File
        If Not Image_Handle Then RuntimeError "[VectorLib]" + Chr(13) + "Tracer_ConvertImageToArray ()" + Chr(13) + "Invalid image handle."
    EndIf 

    Tracer_Image_SizeX = ImageWidth  ( Image_Handle ) + 2  ; Padding.
    Tracer_Image_SizeY = ImageHeight ( Image_Handle ) + 2  ; Padding.
    Dim Tracer_Image( Tracer_Image_SizeX , Tracer_Image_SizeY )

    LockBuffer ImageBuffer ( Image_Handle , Frame )
    ; Center the image to leave padding on all sides (so pixels/image/points are shifted 2 units from 0).
    For PosY = 1 To Tracer_Image_SizeY-2
        For PosX = 1 To Tracer_Image_SizeX-2
            Tracer_Image( PosX+1 , PosY+1 ) = ReadPixelFast ( PosX-1 , PosY-1 , ImageBuffer ( Image_Handle , Frame ) ) And $FFFFFF
        Next
    Next
    UnlockBuffer ImageBuffer ( Image_Handle , Frame )

    If LoadFromFile
        FreeImage Image_Handle
    EndIf 
End Function


Function Tracer_AddPoint.Tracer_Point ( X , Y , CheckIfExist = True )
    Local Point.Tracer_Point

    If ( Not CheckIfExist ) Or ( CheckIfExist And ( Not Tracer_PointInCollection ( X , Y ) ) )
        Point = New Tracer_Point
        Point\X = X
        Point\Y = Y
    End If

    Return Point
End Function


Function Tracer_PointInCollection ( X , Y )
    Local Point.Tracer_Point

    For Point = Each Tracer_Point
        If Point\X = X And Point\Y = Y Then Return True
    Next

    Return False
End Function


Function Tracer_PointIsAlone ( PosX , PosY , MaskRGB )
    Local X
    Local Y

    For Y = -1 To 1
        For X = -1 To 1
            If X <> 0 And Y <> 0 And Tracer_Image( PosX + X , PosY + Y ) <> MaskRGB
                Return False
            End If
        Next
    Next

    Return True
End Function


Function Tracer_TraceContour ( HotSpotX = -1 , HotSpotY = -1 , MaskRGB = 0 , MaskGreen = -1 , MaskBlue = -1 )
    ; Tracing contour begins at the first found occupied (non-transparent) pixel,
    ; starting in the upperleft corner, then going from left to right and top to bottom.
    ; Transparent color is the optionally specified mask color (default is black (0,0,0)).
    ; If MaskGreen AND MaskBlue are specified, MaskRGB will serve as MaskRED.
    Local Start.Tracer_Point  ; Location (absolute).
    Local Pos.Tracer_Point    ; Location (absolute).
    Local Scan.Tracer_Point   ; Vector (relative).
    Local Found.Tracer_Point  ; Location (absolute).
    Local Dir                 ; Direction.

    If MaskGreen <> -1 And MaskBlue <> -1
        MaskRGB = MaskRGB Shl 16 + MaskGreen Shl 8 + MaskBlue
    End If

    ;- Find starting point for tracing.
    Start = Tracer_PointCreate ( 2 , 2 )
    Repeat
        If Tracer_Image( Start\X , Start\Y ) <> MaskRGB
            If Not Tracer_PointIsAlone ( Start\X , Start\Y , MaskRGB )
                Exit
            End If
        End If
        Start\X = Start\X + 1
        If Start\X > Tracer_Image_SizeX - 1
            Start\X = 2
            Start\Y = Start\Y + 1
            If Start\Y > Tracer_Image_SizeY - 1
                RuntimeError "[VectorLib]" + Chr(13) + "Tracer_ConvertContourToPoints ()" + Chr(13) + "Image must contain more data."
            End If
        End If
    Forever

    ;- Orientation.
    If HotSpotX = -1 Or HotSpotY = -1
        ; Points have hotspot at (0,0) by default.
        HotSpotX = Tracer_Image_SizeX / 2
        HotSpotY = Tracer_Image_SizeY / 2
    Else If HotSpotX > Tracer_Image_SizeX-1 Or HotSpotY > Tracer_Image_SizeY-1
        HotSpotX = 1
        HotSpotY = 1
    Else
        HotSpotX = HotSpotX + 1
        HotSpotY = HotSpotY + 1
    End If

    ;- Trace contour.
    ; Current position.
    Pos = Tracer_PointCreate ( Start\X , Start\Y )
    ; (Absolute) spot to be scanned for occupied pixel.
    Found = Tracer_PointCreate ()
    ; Scan direction from current position.
    Dir = Dir_UpLeft
    Repeat
        ; Relative scan vector.
        Scan = DirToVector ( Dir )
        ; Scan this spot for an occupied pixel.
        Found\X = Pos\X + Scan\X
        Found\Y = Pos\Y + Scan\Y
        ; Found an occupied pixel.
        If Tracer_Image( Found\X , Found\Y ) <> MaskRGB
            ; Add this spot to the list.
            Tracer_AddPoint Found\X - HotSpotX , Found\Y - HotSpotY , False
            ; Move to occupied spot.
            Pos\X = Found\X
            Pos\Y = Found\Y
            ; Reverse search direction.
            Dir = VectorToDir ( -Scan\X , -Scan\Y )
        End If
        ; Scan the next (neighboring) pixel by rotating clockwise (8 directions).
        Dir = Dir_RotateRight ( Dir )
        Delete Scan
    Until Found\X = Start\X And Found\Y = Start\Y

    ; Clean up.
    Delete Start
    Delete Pos
    Delete Found
End Function


Function Tracer_OptimizePoints ()
    Local p.Tracer_Point[ 3 ]

    p[ 1 ] = First Tracer_Point
    p[ 2 ] = After p[ 1 ]
    p[ 3 ] = After p[ 2 ]
    Repeat
        If Dir_IsSame ( p )
            Delete p[ 2 ]
            p[ 2 ] = p[ 3 ]
        Else
            If After p[ 2 ] = Null
                p[ 2 ] = First Tracer_Point
            Else
                p[ 2 ] = After p[ 2 ]
            End If
            If After p[ 1 ] = Null
                Exit
            Else
                p[ 1 ] = After p[ 1 ]
            End If
        End If
        If After p[ 3 ] = Null
            p[ 3 ] = First Tracer_Point
        Else
            p[ 3 ] = After p[ 3 ]
        End If
    Forever
End Function

;-------------------------------------------------------------------------------

Const NoMoreData = 1234567890

Const Source_ImageFile   = 0
Const Source_ImageHandle = 1
Const Source_Data        = 2
Const Source_File        = 3

Type VectorModel
    Field Closed  ; Connect last with first point (true/false).
    Field FirstVertex.VectorPoint  ; Root vertex (all vertices are absolute).
End Type

Type VectorPoint  ; Vertex (polar orientation).
    Field Angle#     ; Polar distance (converted from absolute cartesian coordinate X,Y).
    Field Distance#
    Field NextVertex.VectorPoint
    ; Fast access
    Field X#,Y#
End Type


; create an empty vector model
Function VectorModel_Create.VectorModel( ModelClosed = True )
    Local Model.VectorModel

    Model = New VectorModel
    Model\Closed = ModelClosed

    Return Model
End Function


; Change and/or retrieve vector model closed status
Function VectorModel_Closed( Model.VectorModel , ModelClosed = -1 )
    ; -n..-2: toggle closed between True and False
    If ModelClosed < -1
        Model\Closed = Not Model\Closed
    ; +0..+n: set closed to True (non-zero) or False (zero)
    ElseIf ModelClosed > -1
        Model\Closed = ModelClosed <> False 
    EndIf 
    ; return current state by default (always)
    Return Model\Closed
End Function


; add point to model (last in list)
Function VectorModel_AddPoint( Model.VectorModel , PointX#,PointY# )
    Local Point.VectorPoint

    ; no points yet
    If Model\FirstVertex = Null
        Point = New VectorPoint
        Model\FirstVertex = Point
    ; at least 1 point
    Else
        ; walk through all points, till after last point
        Point = Model\FirstVertex
        While Point\NextVertex <> Null
            Point = After Point
        Wend
        ; add a new point
        Point\NextVertex = New VectorPoint
        ; switch to there
        Point = Point\NextVertex
    EndIf
    Point\X = PointX
    Point\Y = PointY
    Point\Angle = ATan2( PointX,PointY )
    Point\Distance = Sqr( PointX*PointX + PointY*PointY )
End Function


Function VectorModel_InsertPoint( Model.VectorModel )
; what about inserting using pointers rather than numbers?
; try a bit of pseudo code for modifying a ship's hull

;    ; no points yet
;    If Model\FirstVertex = Null
;        Point = New VectorPoint
;        Model\FirstVertex = Point
;        Point\Angle = ATan2( PointX,PointY )
;        Point\Distance = Sqr( PointX*PointX + PointY*PointY )
;    ; at least 1 point
;    Else
;        ; first (0) , insert first
;        ; >0 , insert after/before any
;        ; last (n) , append
;    EndIf
End Function


; remove last point from model
Function VectorModel_RemovePoint( Model.VectorModel )
    Local Point.VectorPoint

    ; at least 1 point in model
    If Model\FirstVertex <> Null
        ; start there
        Point = Model\FirstVertex
        ; and walk to last point
        While Point\NextVertex <> Null
            Point = After Point
        Wend
        ; remove point
        Delete Point
    EndIf
End Function


; wipe entire model
Function VectorModel_Delete( Model.VectorModel )
    Local Point.VectorPoint
    Local Trace.VectorPoint

    ; model exists
    If Model = Null Then Return
    ; at least 1 point in model to start a trace
    If Model\FirstVertex <> Null
        ; start at first point
        Point = Model\FirstVertex
        ; trace until none after point
        While Point\NextVertex <> Null
            ; trace follows point
            Trace = Point
            ; point follows next
            Point = After Point
            ; wipe the trace
            Delete Trace
        Wend
        ; wipe what remains
        Delete Point
    EndIf
    ; and its base
    Delete Model
End Function


; clear model contents but keep container
Function VectorModel_Clear( Model.VectorModel , ModelClosed = True )
    Local Point.VectorPoint
    Local Trace.VectorPoint

    ; at least 1 point in model to start a trace
    If Model\FirstVertex <> Null
        ; start at first point
        Point = Model\FirstVertex
        ; trace until none after point
        While Point\NextVertex <> Null
            ; trace follows point
            Trace = Point
            ; point follows next
            Point = After Point
            ; wipe the trace
            Delete Trace
        Wend
        ; wipe what remains
        Delete Point
    EndIf
    ; reset/change model closed status
    Model\Closed = ModelClosed
End Function


Function CreateVectorModel.VectorModel ( Source = -1 , File$ = "" , HotSpotX# = -1 , HotSpotY# = -1 , MaskRed = 0 , MaskGreen = 0 , MaskBlue = 0 , Precision = 1 , Closed = True , Frame = 0 )
    ; Precision means skipping value for points (1=all points(best precision),max(numPoints-2)=2 points total).
    ; Any vector model/image containing less than 3 points will be an illegal vector model for usage.
    Local Model.VectorModel
    Local Image

    If Source = -1 Then Return

    ;- Prepare sources to be used to create vector model and create vector model according to specified source.
    Select Source
        Case Source_ImageFile
            ; Validate file.
            If File = "" Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "No file specified."
            If FileType ( File ) <> 1 Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "File does not exist:" + Chr(13) + File
            ; Store image's contents (pixels) in an array.
            Tracer_ConvertImageToArray File , True  ; load from single image file
            ; Trace image's contour (from array) and store traced contour points.
            Tracer_TraceContour HotSpotX , HotSpotY , MaskRed , MaskGreen , MaskBlue
            ; Validate model point count.
            If Tracer_PointCount () < 3 Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "Model does not contain enough data."
            ; Remove insignificant points (adjacent vertices with equal heading).
            Tracer_OptimizePoints
            ; Transform Blitz structured (dependent) points to independent and mathematically optimized vector model.
            Model = VectorLib_ConvertPointsToVectorModel ( Closed , Precision )
        Case Source_ImageHandle
            ; Validate image handle
            Image = Int File
            If Not Image Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "Invalid image handle."
            ; Store image's contents (pixels) in an array.
            Tracer_ConvertImageToArray Image , False , Frame  ; use existing image handle
            ; Trace image's contour (from array) and store traced contour points.
            Tracer_TraceContour HotSpotX , HotSpotY , MaskRed , MaskGreen , MaskBlue
            ; Validate model point count.
            If Tracer_PointCount () < 3 Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "Model does not contain enough data."
            ; Remove insignificant points (adjacent vertices with equal heading).
            Tracer_OptimizePoints
            ; Transform Blitz structured (dependent) points to independent and mathematically optimized vector model.
            Model = VectorLib_ConvertPointsToVectorModel ( Closed , Precision )
        Case Source_Data
            ; Create vector model from points in data.
            Model = VectorLib_ConvertDataToVectorModel ( Closed , Precision )
        Case Source_File
            ; Validate file.
            If File = "" Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "No file specified."
            If FileType ( File ) <> 1 Then RuntimeError "[VectorLib]" + Chr(13) + "CreateVectorModel ()" + Chr(13) + "File does not exist:" + Chr(13) + File
            ; Create vector model from points in file.
            Model = VectorLib_ImportVectorModel ( File , Closed , Precision )
        Default
            Return
    End Select

    Return Model
End Function


Function VectorLib_ConvertPointsToVectorModel.VectorModel ( ModelClosed = True , ModelPrecision = 1 )
    Local Model.VectorModel
    Local ThisPoint.VectorPoint  ; Pointer to Vertex.
    Local Point.Tracer_Point
    Local Counter
    Local PointX#
    Local PointY#

    Model = New VectorModel
    Model\FirstVertex = New VectorPoint
    Model\Closed = ModelClosed
    ThisPoint = Model\FirstVertex

    For Point = Each Tracer_Point
        Counter = Counter + 1
        If Counter = ModelPrecision
            Counter = 0

            ; Center the point on a pixel (instead of the upperleft corner)
            PointX = Point\X - 0.5
            PointY = Point\Y - 0.5

            ThisPoint\X = Point\X
            ThisPoint\Y = Point\Y

            ; Convert absolute cartesian position to absolute polar distance.
            ThisPoint\Angle = ATan2 ( PointX , PointY )
            ThisPoint\Distance = Sqr ( PointX * PointX + PointY * PointY )

            ; Next Vertex.
            ThisPoint\NextVertex = New VectorPoint
            ThisPoint = ThisPoint\NextVertex
        End If
    Next

    ; Remove empty VectorPoint.
    Delete ThisPoint

    ; Remove traces.
    Delete Each Tracer_Point

    Return Model
End Function


Function VectorLib_ConvertDataToVectorModel.VectorModel ( ModelClosed = True , ModelPrecision = 1 )
    Local Model.VectorModel
    Local ThisPoint.VectorPoint  ; Pointer to Vertex.
    Local PointX
    Local PointY
    Local Counter

    Model = New VectorModel
    Model\FirstVertex = New VectorPoint
    Model\Closed = ModelClosed
    ThisPoint = Model\FirstVertex

    Repeat
        Read PointX
        If PointX = NoMoreData Then Exit
        Read PointY

        Counter = Counter + 1
        If Counter = ModelPrecision
            Counter = 0

            ThisPoint\X = PointX
            ThisPoint\Y = PointY

            ; Convert absolute cartesian position to absolute polar distance.
            ThisPoint\Angle = ATan2 ( PointX , PointY )
            ThisPoint\Distance = Sqr ( PointX * PointX + PointY * PointY )

            ; Next Vertex.
            ThisPoint\NextVertex = New VectorPoint
            ThisPoint = ThisPoint\NextVertex
        End If
    Forever

    ; Remove empty VectorPoint.
    Delete ThisPoint

    Return Model
End Function


Function VectorLib_ImportVectorModel.VectorModel ( File$ , ModelClosed = True , ModelPrecision = 1 )
    ; The file format is a simple list of pairs of 4 byte signed long integers for X and Y points, respectively.
    Local Model.VectorModel
    Local ThisVertex.VectorPoint
    Local PointX
    Local PointY
    Local Counter
    Local File_Handle

    Model = New VectorModel
    Model\FirstVertex = New VectorPoint
    Model\Closed = ModelClosed
    ThisVertex = Model\FirstVertex
    File_Handle = OpenFile ( File )

    Repeat
        PointX = ReadInt ( File_Handle )
        If Eof ( File_Handle ) Then Exit
        PointY = ReadInt ( File_Handle )

        Counter = Counter + 1
        If Counter = ModelPrecision
            Counter = 0

            ThisVertex\X = PointX
            ThisVertex\Y = PointY

            ; Convert absolute cartesian position to absolute polar distance.
            ThisVertex\Angle = ATan2 ( PointX , PointY )
            ThisVertex\Distance = Sqr ( PointX * PointX + PointY * PointY )

            ; Next Vertex.
            ThisVertex\NextVertex = New VectorPoint
            ThisVertex = ThisVertex\NextVertex
        End If
    Forever

    CloseFile File_Handle

    ; Remove empty VectorPoint.
    Delete ThisVertex

    Return Model
End Function


Function VectorLib_PointInsideVectorModel ( Model.VectorModel , ModelX , ModelY , ModelAngle , PointX# , PointY# , ModelScale# = 1 )
    Local ThisPoint.VectorPoint  ; Vertices
    Local NextPoint.VectorPoint
    Local ThisX#
    Local ThisY#
    Local NextX#
    Local NextY#
    Local PointInVectorModel

    ; ThisPoint starts at 1st vertex.
    ThisPoint = Model\FirstVertex
    ; NextPoint starts at 2nd vertex.
    NextPoint = Model\FirstVertex\NextVertex
    PointInVectorModel = False

    Repeat
        ;- Point in VectorModel algorithm (headache included).
        ThisX = Sin ( ThisPoint\Angle + ModelAngle ) * ThisPoint\Distance * ModelScale + ModelX
        ThisY = Cos ( ThisPoint\Angle + ModelAngle ) * ThisPoint\Distance * ModelScale + ModelY
        NextX = Sin ( NextPoint\Angle + ModelAngle ) * NextPoint\Distance * ModelScale + ModelX
        NextY = Cos ( NextPoint\Angle + ModelAngle ) * NextPoint\Distance * ModelScale + ModelY
        If PointX < ( (NextX - ThisX) * (PointY - ThisY) / (NextY - ThisY) + ThisX )
            If ( ThisY <= PointY And PointY < NextY ) Or ( NextY <= PointY And PointY < ThisY )
                PointInVectorModel = Not PointInVectorModel
            End If
        End If

        If NextPoint\NextVertex = Null  ; If no more vertices.
            NextPoint = Model\FirstVertex  ; Start at first vertex again.
        Else
            NextPoint = NextPoint\NextVertex  ; Otherwise goto next vertex.
        End If

        ThisPoint = ThisPoint\NextVertex  ; Goto next vertex.
    Until ThisPoint = Null

    Return PointInVectorModel
End Function


Function VectorModelsCollide ( Model1.VectorModel , Model1_PosX , Model1_PosY , Model1_Angle , Model2.VectorModel , Model2_PosX , Model2_PosY , Model2_Angle , Model1_Scale# = 1 , Model2_Scale# = 1 )
    Local ThisPoint.VectorPoint  ; Vertex.
    Local PointX#
    Local PointY#

    ThisPoint = Model1\FirstVertex
    Repeat
        PointX = Sin ( ThisPoint\Angle - Model1_Angle ) * ThisPoint\Distance * Model1_Scale + Model1_PosX
        PointY = Cos ( ThisPoint\Angle - Model1_Angle ) * ThisPoint\Distance * Model1_Scale + Model1_PosY
        If VectorLib_PointInsideVectorModel ( Model2 , Model2_PosX , Model2_PosY , Model2_Angle , PointX , PointY , Model2_Scale ) Then Return True

        ThisPoint = ThisPoint\NextVertex
    Until ThisPoint = Null

    ThisPoint = Model2\FirstVertex
    Repeat
        PointX = Sin ( ThisPoint\Angle - Model2_Angle ) * ThisPoint\Distance * Model2_Scale + Model2_PosX
        PointY = Cos ( ThisPoint\Angle - Model2_Angle ) * ThisPoint\Distance * Model2_Scale + Model2_PosY
        If VectorLib_PointInsideVectorModel ( Model1 , Model1_PosX , Model1_PosY , Model1_Angle , PointX , PointY , Model1_Scale ) Then Return True

        ThisPoint = ThisPoint\NextVertex
    Until ThisPoint = Null

    Return False
End Function


; draw vector model with lines only (use color)
Function DrawVectorModel ( Model.VectorModel , ModelX , ModelY , ModelAngle = 0 , ModelScale# = 1 )
    Local ThisPoint.VectorPoint  ; Vertices.
    Local PrevPoint.VectorPoint

    If Not Model\Closed  ; Starting first(p) -> second(t).
        PrevPoint = Model\FirstVertex
        ThisPoint = Model\FirstVertex\NextVertex
    Else  ; Starting last(p) -> first(t), then first(p) -> second(t).
        ;- Find last vertex.
        PrevPoint = Model\FirstVertex
        Repeat
            PrevPoint = PrevPoint\NextVertex
        Until PrevPoint\NextVertex = Null

        ThisPoint = Model\FirstVertex
    End If

    ;- Iterate through vertices.
    Repeat
        ;- Calculate line coordinates and draw.
        ThisX = Sin ( ThisPoint\Angle - ModelAngle ) * ThisPoint\Distance * ModelScale + ModelX
        ThisY = Cos ( ThisPoint\Angle - ModelAngle ) * ThisPoint\Distance * ModelScale + ModelY
        PrevX = Sin ( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelX
        PrevY = Cos ( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelY

        Line ThisX,ThisY , PrevX,PrevY  ; Use Line for speed, dLine for no bugs (See dLine).

        ;- PrevPoint is ThisPoint's shadow.
        PrevPoint = ThisPoint

        ;- ThisPoint becomes the next vertex.
        ThisPoint = ThisPoint\NextVertex
    Until ThisPoint = Null
End Function


; draw vector model with thick lines (use color)
Function DrawVectorModelThick( Model.VectorModel , ModelX , ModelY , ModelAngle , ModelScale# = 1 )
    Local ThisPoint.VectorPoint  ; Vertices.
    Local PrevPoint.VectorPoint

    If Not Model\Closed  ; Starting first(p) -> second(t).
        PrevPoint = Model\FirstVertex
        ThisPoint = Model\FirstVertex\NextVertex
    Else  ; Starting last(p) -> first(t), then first(p) -> second(t).
        ;- Find last vertex.
        PrevPoint = Model\FirstVertex
        Repeat
            PrevPoint = PrevPoint\NextVertex
        Until PrevPoint\NextVertex = Null

        ThisPoint = Model\FirstVertex
    End If

    ;- Iterate through vertices.
    Repeat
        ;- Calculate line coordinates and draw.
        ThisX = Sin ( ThisPoint\Angle - ModelAngle ) * ThisPoint\Distance * ModelScale + ModelX
        ThisY = Cos ( ThisPoint\Angle - ModelAngle ) * ThisPoint\Distance * ModelScale + ModelY
        PrevX = Sin ( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelX
        PrevY = Cos ( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelY

        Line ThisX,ThisY , PrevX,PrevY
        Line PrevX,PrevY , ThisX,ThisY

        ;- PrevPoint is ThisPoint's shadow.
        PrevPoint = ThisPoint

        ;- ThisPoint becomes the next vertex.
        ThisPoint = ThisPoint\NextVertex
    Until ThisPoint = Null
End Function


; draw model with outline (5 lines per line) with custom color
Function DrawVectorModelOutline( Model.VectorModel , ModelX,ModelY , ModelAngle,ModelScale#=1 , fr=255,fg=255,fb=255 , br=128,bg=128,bb=128 )
    Local ThisPoint.VectorPoint  ; Vertices.
    Local PrevPoint.VectorPoint

    If Not Model\Closed  ; Starting first(p) -> second(t).
        PrevPoint = Model\FirstVertex
        ThisPoint = Model\FirstVertex\NextVertex
    Else  ; Starting last(p) -> first(t), then first(p) -> second(t).
        ;- Find last vertex.
        PrevPoint = Model\FirstVertex
        Repeat
            PrevPoint = PrevPoint\NextVertex
        Until PrevPoint\NextVertex = Null

        ThisPoint = Model\FirstVertex
    End If

    ;- Iterate through vertices.
    Repeat
        ;- Calculate line coordinates and draw.
        ThisX = Sin ( ThisPoint\Angle - ModelAngle ) * ThisPoint\Distance * ModelScale + ModelX
        ThisY = Cos ( ThisPoint\Angle - ModelAngle ) * ThisPoint\Distance * ModelScale + ModelY
        PrevX = Sin ( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelX
        PrevY = Cos ( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelY

        Color br,bg,bb
        Line ThisX  ,ThisY-1 , PrevX  ,PrevY-1
        Line ThisX  ,ThisY+1 , PrevX  ,PrevY+1
        Line ThisX-1,ThisY   , PrevX-1,PrevY
        Line ThisX+1,ThisY   , PrevX+1,PrevY
        Color fr,fg,fb
        Line ThisX,ThisY , PrevX,PrevY

        ;- PrevPoint is ThisPoint's shadow.
        PrevPoint = ThisPoint

        ;- ThisPoint becomes the next vertex.
        ThisPoint = ThisPoint\NextVertex
    Until ThisPoint = Null
End Function


; draw vector model with points only (use color)
Function DrawVectorModelPoints( Model.VectorModel , ModelX,ModelY , ModelAngle=0,ModelScale#=1 , PointSize=2 )
    Local PrevPoint.VectorPoint  ; Vertices.
    Local HalfPointSize

    HalfPointSize = PointSize / 2
    PrevPoint = Model\FirstVertex
    ;- Iterate through vertices.
    Repeat
        ;- Calculate line coordinates and draw.
        PrevX = Sin( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelX
        PrevY = Cos( PrevPoint\Angle - ModelAngle ) * PrevPoint\Distance * ModelScale + ModelY

        ;  Note: Use Line for speed, dLine for no bugs (See dLine).
        Rect PrevX-HalfPointSize,PrevY-HalfPointSize , PointSize,PointSize

        ;- ThisPoint becomes the next vertex.
        PrevPoint = PrevPoint\NextVertex
    Until PrevPoint = Null
End Function


Function DetectVectorModelCollisionRadius ( Model.VectorModel )
    ; Use the returned radius as a minimum collision distance before testing for precise collisions.
    Local MaxDistance#
    Local Vertex.VectorPoint

    Vertex = Model\FirstVertex
    Repeat
        If Vertex\Distance > MaxDistance Then MaxDistance = Vertex\Distance
        Vertex = Vertex\NextVertex
    Until Vertex = Null

    Return MaxDistance
End Function


Function VectorModelWidth#( Model.VectorModel )
    Local Min#=2147483647,Max#=-2147483648
    Local Vertex.VectorPoint

    Vertex = Model\FirstVertex
    Repeat
        If Vertex\X > Max Then Max = Vertex\X
        If Vertex\X < Min Then Min = Vertex\X
        Vertex = Vertex\NextVertex
    Until Vertex = Null

    Return Max-Min
End Function


Function VectorModelHeight#( Model.VectorModel )
    Local Min#=2147483647,Max#=-2147483648
    Local Vertex.VectorPoint

    Vertex = Model\FirstVertex
    Repeat
        If Vertex\Y > Max Then Max = Vertex\Y
        If Vertex\Y < Min Then Min = Vertex\Y
        Vertex = Vertex\NextVertex
    Until Vertex = Null

    Return Max-Min
End Function


; Count total number of vertices present in vector model
Function CountVectorModelPoints( Model.VectorModel )
    Local Count
    Local Vertex.VectorPoint

    Count = 0
    Vertex = Model\FirstVertex
    While Vertex <> Null
        Count = Count + 1
        Vertex = Vertex\NextVertex
    Wend

    Return Count
End Function


Function dLine(x1#,y1#,x2#,y2#)
    ; Found a new bug in Blitz where Line using *very* small values will start aliasing, causing jitter and various side effects.
    ; Working around the bug using this routine instead, of which the original was created by The Great Antony :)
    Local steps,xI#
    x2 = x2-x1
    y2 = y2-y1
    If Abs(x2)>Abs(y2) steps = Abs(x2) Else steps = Abs(y2)
    xI = x2 / steps
    y2 = y2 / steps
    For x2 = 1 To steps
        Plot x1,y1
        x1=x1+xI:y1=y1+y2
    Next
End Function

