; original from robotix (version 1o+)
; functions:
; - StarField_Generate
; - StarField_Move vx#,vy#
; - StarField_Display

Const ScreenSizeX = sx
Const ScreenSizeY = sy


Const StarField_Depth = 8
Const StarField_Pixels = 12
Const StarField_MaxFlux = 24
Const StarField_Darkness = StarField_MaxFlux
Const StarField_Brightness = 192-StarField_MaxFlux


Type STAR
    Field PosX#
    Field PosY#
End Type


Global Star.STAR


Function Star_Create.STAR ( PosX , PosY )
    Star = New STAR
    Star\PosX = PosX
    Star\PosY = PosY
    Return Star
End Function


Dim StarField.STAR( StarField_Depth , StarField_Pixels )


Function StarField_Generate ()
    For Depth = 1 To StarField_Depth
        For PosX = 0 To StarField_Pixels - 1
            Min_PosX = PosX * ScreenSizeX / StarField_Pixels
            Max_PosX = (PosX-1) * ScreenSizeX / StarField_Pixels - 1
            Star_PosX = Rand ( Min_PosX , Max_PosX )
            Star_PosY = Rand ( 0 , ScreenSizeY - 1 )
            StarField( Depth , PosX + 1 ) = Star_Create ( Star_PosX , Star_PosY )
        Next
    Next
End Function


Function StarField_Move ( MoveX# , MoveY# )
    ; limit motion
    If Abs( MoveX ) > ScreenSizeX Then MoveX = ScreenSizeX * Sgn( MoveX )
    If Abs( MoveY ) > ScreenSizeY Then MoveY = ScreenSizeY * Sgn( MoveY )
    ; brightness layer
    For Depth = 1 To StarField_Depth
        ; motion
        AddX# = MoveX / (StarField_Depth - Depth+1)
        AddY# = MoveY / (StarField_Depth - Depth+1)
        ; star
        For Pixel = 1 To StarField_Pixels
            ; move x
            StarField( Depth , Pixel )\PosX = StarField( Depth , Pixel )\PosX + AddX
            ; check limit x left
            If Int StarField( Depth , Pixel )\PosX <= -1
                ; move x right
                StarField( Depth , Pixel )\PosX = StarField( Depth , Pixel )\PosX + ScreenSizeX
                ; rand y
                StarField( Depth , Pixel )\PosY = Rand( 0,ScreenSizeY-1 )
            EndIf
            ; check limit x right
            If Int StarField( Depth , Pixel )\PosX >= ScreenSizeX+1
                ; move x left
                StarField( Depth , Pixel )\PosX = StarField( Depth , Pixel )\PosX - ScreenSizeX
                ; rand y
                StarField( Depth , Pixel )\PosY = Rand( 0,ScreenSizeY-1 )
            EndIf
            ; move y
            StarField( Depth , Pixel )\PosY = StarField( Depth , Pixel )\PosY + AddY
            ; check limit y top
            If Int StarField( Depth , Pixel )\PosY <= -1
                ; move y down
                StarField( Depth , Pixel )\PosY = StarField( Depth , Pixel )\PosY + ScreenSizeY
                ; rand x
                StarField( Depth , Pixel )\PosX = Rand( 0,ScreenSizeX-1 )
            EndIf
            ; check limit y bottom
            If Int StarField( Depth , Pixel )\PosY >= ScreenSizeY+1
                ; move y up
                StarField( Depth , Pixel )\PosY = StarField( Depth , Pixel )\PosY - ScreenSizeY
                ; rand x
                StarField( Depth , Pixel )\PosX = Rand( 0,ScreenSizeX-1 )
            EndIf
        Next
    Next
End Function


Function StarField_Display ()
    ; brightness range
    Brightness_BaseLevel = StarField_Brightness - StarField_Darkness
    ; brightness layer
    For Depth = 1 To StarField_Depth
        ; display brightness: brightness of layer per depth with darkness offset
        ColorValue = (Depth-1) * Brightness_BaseLevel / (StarField_Depth-1) + StarField_Darkness
        ; star
        For Pixel = 1 To StarField_Pixels
            c = ColorValue + Rand( -StarField_MaxFlux,StarField_MaxFlux )
            Color c,c,c
            Plot StarField( Depth , Pixel )\PosX , StarField( Depth , Pixel )\PosY
        Next
    Next
End Function


;-- self-initialize

starfield_generate

